namespace Thtoeo
{
	using System;
	using System.Collections.Generic;
	using System.Linq;
	using UnityEngine;

	public class MorphHistory : MVRScript
	{
		internal class MorphChange
		{
			internal Guid Id;
			internal float PrevValue;
			internal float NewValue;
			internal bool Reverted;

			internal MorphChange(Guid id, float prevValue, float newValue)
			{
				Id = id;
				PrevValue = prevValue;
				NewValue = newValue;
			}
		}

		internal Dictionary<Guid, DAZMorph> MorphLookup = new Dictionary<Guid, DAZMorph>();
		internal Dictionary<Guid, float> ActiveMorphValues = new Dictionary<Guid, float>();
		internal IList<MorphChange> MorphChanges = new List<MorphChange>();

		public override void Init()
		{
			try
			{
				var morphControl = (containingAtom.GetStorableByID("geometry") as DAZCharacterSelector)?.morphsControlUI;
				
				if (morphControl == null)
				{
					throw new Exception("Morph control missing: " + containingAtom.uid);
				}

				foreach (var morph in morphControl.GetMorphs())
				{
					var guid = Guid.NewGuid();

					MorphLookup.Add(guid, morph);
					ActiveMorphValues.Add(guid, morph.morphValue);
				}
			}
			catch (Exception e)
			{
				SuperController.LogError("Exception caught: " + e);
			}
		}
		
		void OnDestroy()
		{
			MorphChanges = new List<MorphChange>();
			ActiveMorphValues = new Dictionary<Guid, float>();
			MorphChanges = new List<MorphChange>();
		}

		void Start()
		{
			try
			{
			}
			catch (Exception e)
			{
				SuperController.LogError("Exception caught in Start(): " + e);
			}
		}

		void Update()
		{
			try
			{
				if (Input.GetKey(KeyCode.LeftControl)) 
				{
					if (Input.GetKeyDown(KeyCode.Z))
					{
						// Revert newest non-reverted change
						var lastChange = MorphChanges.LastOrDefault(x => !x.Reverted);
						
						if (lastChange != null)
						{
							MorphLookup[lastChange.Id].morphValue = lastChange.PrevValue;
							ActiveMorphValues[lastChange.Id] = lastChange.PrevValue;
							lastChange.Reverted = true;
						}
					
						return;
					}
					
					if (Input.GetKeyDown(KeyCode.Y))
					{
						// Revert oldest reverted change
						var lastChange = MorphChanges.FirstOrDefault(x => x.Reverted);

						if (lastChange != null)
						{
							MorphLookup[lastChange.Id].morphValue = lastChange.NewValue;
							ActiveMorphValues[lastChange.Id] = lastChange.NewValue;
							lastChange.Reverted = false;
						}
					
						return;
					}
				}
				
				foreach (var morph in MorphLookup)
				{
					if (morph.Value.isDriven)
					{
						continue;
					}
					
					var prevValue = ActiveMorphValues[morph.Key];
					var newValue = morph.Value.morphValue;
					
					if (Math.Abs(prevValue - newValue) > 0.0001)
					{
						// Clear reverted changes when new change happens
						MorphChanges = MorphChanges.Where(x => !x.Reverted).ToList();
						
						var lastChange = MorphChanges.LastOrDefault();

						if (lastChange != null && lastChange.Id == morph.Key)
						{
							lastChange.NewValue = newValue;
						}
						else
						{
							MorphChanges.Add(new MorphChange(morph.Key, prevValue, newValue));
						}
						
						ActiveMorphValues[morph.Key] = newValue;
					}
				}

			}
			catch (Exception e)
			{
				SuperController.LogError("Exception caught in Update(): " + e);
			}
		}
		
		void FixedUpdate()
		{
			try
			{
			}
			catch (Exception e)
			{
				SuperController.LogError("Exception caught: " + e);
			}
		}
	}

}